// engine.h

// How I built this:
// I glued together the churrera routines and the phantomas engine routines I needed.
// The blending was somewhat tricky, as the main player in churrera uses fixed point
// values (subpixel precission) and Phantomas uses pixel values. It wasn't that easy,
// specially making the main player to be able to open gates in any direction. But
// it was fun. 

// Then I modified all the routines for space (and sometimes, for speed). 
// Almost rewrote completely the enemy AI (if you can call "that" AI).
// Brand new to this engine is the frigoababol (Sir ababol turns into
// an ice cube), and the ability to destroy Icy blocks (that's great!).
// The boss is also new.

// Stay tuned. This will be the base engine I'll build "Lala the Magical" upon.
// I'll remove the Icy stuff and add variable sized levels with tons of tilesets.
// "Lala the Magical" will be a BIG game.

// I'm sorry comments are scarce, but I was in a big hurry. I mean, I don't have deadlines,
// but, just like Hannibal Smith, when I'm "on the jazz" I just can't wait for the game
// to be finished. Blame the A-Team.

// Yeah, my favourite gaming platform is/was the Sega Master System.

// Oh, and I lost 25% of my job in a drive crash around July 2012 so I abandoned
// this game completely. Now (April 2013) I'm finishing it.

// The game was abandoned once again. Now (April 2014) we are finishing it FOR GOOD.
// First thing we did is change the movement routines for the new ones as found in
// Churrera 5 (Ninjajar!) which made this game run faster.

// 0 1 2 3 + facing: walk, 1 = stand. 8 + facing = jump/fall
unsigned char *player_frames [] = {
	spr_0, spr_1, spr_2, spr_1,
	spr_4, spr_5, spr_6, spr_5,
	spr_3, spr_7
};

// Functions

unsigned char rand (void) {
	#asm
		.rnd 
			ld  hl,0xA280
	        ld  de,0xC0DE
	        ld  a,h         ; t = x ^ (x << 1)
	        add a,a
	        xor h
	        ld  h,l         ; x = y
	        ld  l,d         ; y = z
	        ld  d,e         ; z = w
	        ld  e,a
	        rra             ; t = t ^ (t >> 1)
	        xor e
	        ld  e,a
	        ld  a,d         ; w = w ^ ( w << 3 ) ^ t
	        add a,a
	        add a,a
	        add a,a
	        xor d
	        xor e
	        ld  e,a
	        ld  (rnd+1),hl
	        ld  (rnd+4),de
	        ld	(_randres), a
	#endasm
	return randres [0];
}

void srand (void) {
	#asm
			ld	hl, (_seed1)
			ld	(rnd+1),hl
			ld	hl, (_seed2)			
			ld	(rnd+4),hl
	#endasm
}

void step () {
	#asm
		ld a, 16
		out (254), a
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		xor 16
		out (254), a
	#endasm	
}

void decrease_life () {
	if (p_life > 0) {
		p_life --;
	} 		
}

void __FASTCALL__ attr (signed char x, signed char y) {
	return map_attr [x + (y << 4) - y];
}

void __FASTCALL__ tileat (signed char x, signed char y) {
	return map_buff [x + (y << 4) - y];
}

void init_game () {
	for (it = 0; it < 60; it ++) {
		men [it] = 3;
	}
		
	p_x = PLAYER_INI_X << 10;
	p_y = PLAYER_INI_Y << 10;
	p_vy = 0;
	p_vx = 0;
	p_cont_salto = 0;
	p_saltando = 0;
	p_frame = 0;
	p_subframe = 0;
	p_facing = 1;
	p_objs = 0;
	p_state = EST_NORMAL;
	p_counter = 0;
	p_swording = 0;
	
	p_g = PLAYER_G; 	
	p_ax = PLAYER_AX;
	p_rx = PLAYER_RX;
	p_axf = PLAYER_AXF;
	p_rxf = PLAYER_RXF;
	p_salto = PLAYER_VY_INICIAL_SALTO;
	p_incr_salto = PLAYER_INCR_SALTO;
	
	p_next_frame = spr_1;
	
	s_on = 0;
	s_type = 0;
	s_next_frame = spr_19;
}

unsigned char do_process_breakable;

void delete_tile (unsigned char x, unsigned char y) {
	if (x < 15) {
		if (attr (x, y) & 16) {
			do_process_breakable = 1;	
			for (it = 0; it < MAX_BREAKABLE; it ++) {
				if (b_f [it] == 0)
					break;	
			}
			b_f [it] = 1;
			b_x [it] = x;
			b_y [it] = y;
			set_map_tile (b_x [it], b_y [it], 0);
			map_attr [x + (y << 4) - y] = 8;
			draw_map_tile_inv (x, y, 17);
			cpc_WyzStartEffect (WYZ_CANAL_EFECTOS, 6);
		}
	}
}

void process_breakable () {
	do_process_breakable = 0;
	for (it = 0; it < MAX_BREAKABLE; it ++) {
		if (b_f [it] > 0) {
			b_f [it] ++;
			if (b_f [it] == MAX_BREAKABLE_FRAMES) {
				b_f [it] = 0;
				tget = 0;
				if ((rand () & FREQ_COIN) == 1) tget = 15;
				if ((rand () & FREQ_LIFE) == 1) tget = 16;
				map_attr [b_x [it] + (b_y [it] << 4) - b_y [it]] = 0;
				map_buff [b_x [it] + (b_y [it] << 4) - b_y [it]] = tget;
				draw_map_tile_inv (b_x [it], b_y [it], tget);
				cpc_WyzStartEffect (WYZ_CANAL_EFECTOS, 0);
			} else {
				do_process_breakable = 1;
			}
		}	
	}	
}


unsigned char swoffs_x [] = {8, 10, 12, 14, 15, 15, 14, 13, 10};
unsigned char swoffs_y [] = {2,  2,  2, 3,  4,  4,  5,  6,  7};
void __FASTCALL__ render_sword (void) {
	py = (p_y >> 6);
	px = (p_x >> 6);
		
	if (s_type == SWORD_TYPE_UP) {
		s_x = px + swoffs_y [s_frame];
		s_y = py + 7 - swoffs_x [s_frame];
		if (s_frame > 2 && s_frame < 6) {
			delete_tile ((s_x >> 4) + ((s_x & 15) > 11), (s_y >> 4));
		}
	} else {
		s_y = py + swoffs_y [s_frame];
		if (s_type == SWORD_TYPE_LEFT) {
			s_x = px + 6 - swoffs_x [s_frame];
			if (s_frame > 2 && s_frame < 6) {
				delete_tile (s_x >> 4, (s_y >> 4) + ((s_y & 15) > 11));
			} 
		} else {
			s_x = (p_x >> 6) + swoffs_x [s_frame];
			if (s_frame > 2 && s_frame < 6) {
				delete_tile (((s_x + 6) >> 4), (s_y >> 4) + ((s_y & 15) > 11));
			}
		}
	}
				
	s_frame ++;
	
	if (s_frame == 9) {
		s_on = 0;
		p_swording = 0;
	}	
	
	// CPC: cuidao con salirse!!
	if (s_x < 0) s_x = 0;
	if (s_x > 224) s_x = 224;
	if (s_y < 0) s_y = 0;
}

void __FASTCALL__ get_from (unsigned char x, unsigned char y) {
	// gets object from x, y.
	tget = TILEAT (x, y);
	switch (tget) {
		case 14:
			p_keys ++;
			cpc_WyzStartEffect (WYZ_CANAL_EFECTOS, 9);
			break;
		case 15:
			if (p_score < 255)	p_score ++;
			cpc_WyzStartEffect (WYZ_CANAL_EFECTOS, 9);
			if (p_score == 25) {
				p_score = 0;
				p_life ++;
				cpc_WyzStartEffect (WYZ_CANAL_EFECTOS, 5);
			}
			break;
		case 16:
			p_life ++;
			cpc_WyzStartEffect (WYZ_CANAL_EFECTOS, 5);
			break;
	}
	
	TILEAT (x, y) = 0;
	//TILE (x, y) = 0;
	set_map_tile (x, y, 0);
	draw_map_tile_inv (x, y, 0);
}

void __FASTCALL__ open_gate_recursive (unsigned char x, unsigned char y) {
	if (TILEAT (x, y) == 13) {
		TILEAT (x, y) = 0;
		//TILE (x, y) = 0;
		set_map_tile (x, y, 0);
		map_attr [x + (y << 4) - y] = 0;
		draw_map_tile_inv (x, y, 0);
		
		open_gate_recursive (x - 1, y);
		open_gate_recursive (x + 1, y);
		open_gate_recursive (x, y - 1);
		open_gate_recursive (x, y + 1);
	}
}

unsigned char __FASTCALL__ open_gate (unsigned char x, unsigned char y) {
	if (TILEAT(x, y) == 13 && p_keys > 0) {
		p_keys --;
		open_gate_recursive (x, y);
		return 1;
	}
	return 0;
}

unsigned char possee, pressjump;
unsigned char ptx1, ptx2, ptx3, pty1, pty2, pty3, pt1, pt2, hit_v, hit_h;
void __FASTCALL__ move (void) {
	if (p_vy < PLAYER_MAX_VY_CAYENDO)
		p_vy += p_g;
	else
		p_vy = PLAYER_MAX_VY_CAYENDO;

	p_y += p_vy;
	
	if (p_y < 0) p_y = 0;
	if (p_y > 9216)	p_y = 9216;

	gpx = p_x >> 6;
	gpy = p_y >> 6;
	ptx1 = (gpx + 4) >> 4;
	ptx2 = (gpx + 11) >> 4;
	ptx3 = (gpx + 8) >> 4;
	pty1 = (gpy + 8) >> 4;
	pty2 = (gpy + 15) >> 4;
	hit_v = 0;
	
	// Vertical collision
	if (p_vy < 0) {
		if ((attr (ptx1, pty1) & 8) || (attr (ptx2, pty1) & 8)) {
			p_vy = 0;
			p_y = ((pty1 + 1) << 10) - 512;
			open_gate (ptx3, pty1);
		} else if ((pt1 & 1) || (pt2 & 1)) {
			hit_v = 1;
		}
	} else if (p_vy > 0 && ((gpy - 1) & 15) < 8) {
		if ((attr (ptx1, pty2) & 12) || (attr (ptx2, pty2) & 12)) {
			if (!((attr (ptx1, pty2 - 1) & 8) || (attr (ptx2, pty2 - 1) & 8))) {
				p_vy = 0;
				p_y = ((pty2 - 1) << 10);
				open_gate (ptx3, pty2);
			}
		} else if ((pt1 & 1) || (pt2 & 1)) {
			hit_v = 1;
		}
	}
	
	// Possee
	gpy = p_y >> 6;
	pty3 = (gpy + 16) >> 4;
	possee = ((attr (ptx1, pty3) & 12) || (attr (ptx2, pty3) & 12));
	
	// Slippery?
	if (attr (ptx1, pty3) & 16 || attr (ptx2, pty3) & 16) {
		pax = p_axf;
		prx = p_rxf;
		maxx = PLAYER_MAX_VXF;
	} else {
		pax = p_ax;
		prx = p_rx;
		maxx = PLAYER_MAX_VX;
	}

	if (cpc_TestKey (KEY_SAL) && p_saltando == 0 && possee) {
		p_saltando = 1;
		p_cont_salto = 0;
		cpc_WyzStartEffect (WYZ_CANAL_EFECTOS, 11);
		p_x = (p_x >> 8) << 8;
	}
	
	if (cpc_TestKey (KEY_SAL) && p_saltando) {
		p_vy -= (PLAYER_VY_INICIAL_SALTO + PLAYER_INCR_SALTO - (p_cont_salto>>1));
		if (p_vy < -PLAYER_MAX_VY_SALTANDO) p_vy = -PLAYER_MAX_VY_SALTANDO;
		p_cont_salto ++;
		if (p_cont_salto == 6)
			p_saltando = 0;
	}
	
	if (0 == cpc_TestKey (KEY_SAL))
		p_saltando = 0;

	if ( ! (cpc_TestKey (KEY_IZQ) || cpc_TestKey (KEY_DER))) {
		if (p_vx > 0) {
			p_vx -= prx;
			if (p_vx < 0) {
				p_x = (p_x >> 8) << 8;
				p_vx = 0;
			}
		} else if (p_vx < 0) {
			p_vx += prx;
			if (p_vx > 0){
				p_x = (p_x >> 8) << 8;
				p_vx = 0;
			}
		}
	}

	if (cpc_TestKey (KEY_IZQ))
		if (p_vx > -maxx) {
			p_facing = 1;
			p_vx -= pax;
		}

	if (cpc_TestKey (KEY_DER))
		if (p_vx < maxx) {
			p_vx += pax;
			p_facing = 0;
		}

	p_x = p_x + p_vx;
	
	// Safe

	if (p_x < 0)		p_x = 0;
	if (p_x > 14336)	p_x = 14336;

	gpx = p_x >> 6;
	gpy = p_y >> 6;
	ptx1 = (gpx + 4) >> 4;
	ptx2 = (gpx + 11) >> 4;
	pty1 = (gpy + 8) >> 4;
	pty2 = (gpy + 15) >> 4;
	hit_h = 0;

	// Horizontal collision
	if (p_vx < 0) {
		pt1 = attr (ptx1, pty1);
		pt2 = attr (ptx1, pty2);
		if ((pt1 & 8) || (pt2 & 8)) {
			p_vx = 0;
			p_x = ((ptx1 + 1) << 10) - 256;
			open_gate (ptx1, pty1);
		} else if ((pt1 & 1) || (pt2 & 1)) {
			hit_h = 1;
		}
	} else if (p_vx > 0) {	
		pt1 = attr (ptx2, pty1);
		pt2 = attr (ptx2, pty2);
		if ((pt1 & 8) || (pt2 & 8)) {
			p_vx = 0;
			p_x = ((ptx2 - 1) << 10) + 256;
			open_gate (ptx2, pty1);
		} else if ((pt1 & 1) || (pt2 & 1)) {
			hit_h = 1;
		}
	}

	gpx = p_x >> 6;
	
	// Objects
	hpx = (gpx + 8) >> 4;
	hpy = (gpy + 8) >> 4;
	if (TILEAT (hpx, hpy) > 13) get_from (hpx, hpy);
	
	// Evil tile
	if (p_state == EST_NORMAL && (hit_v || hit_h)) {
		if (p_vy >= 0) {
			p_vy = -PLAYER_MAX_VX;
	 	} else {
			p_vy = PLAYER_MAX_VX;
		}
		decrease_life ();
		p_state = EST_PARP;
		p_counter = COUNTER_ABABOL_PARP;

		p_y = (p_y >> 10) << 10;
		gpy = p_y >> 6;
		pyy = gpy >> 4;
	}
	
	ptx1 = (gpx + 8) >> 4;
	pty1 = (gpy + 8) >> 4;
	if (attr (ptx1, pty1) == 2) {
		if (p_life > 0 && p_state == EST_NORMAL) {
			////peta_el_beeper (4);
			decrease_life ();
			p_state = EST_PARP;
			p_counter = COUNTER_ABABOL_PARP;
		}
	}
	
	// Update pixel coords
	p_px = p_x >> 6;
	p_py = p_y >> 6;

	// Ice / not ice
	
	if (p_state == EST_ICE) {
		// Frigoababol
		p_next_frame = spr_16;
		if (cpc_TestKey (KEY_FIR)) {
			cpc_WyzStartEffect (WYZ_CANAL_EFECTOS, 10);
			p_vy += ((rand () % 3) << 6) - 64;
			p_vx += ((rand () % 3) << 6) - 64;
			if (p_counter > 0) p_counter --;
		}
		if (p_counter == 0) {
			p_state = EST_PARP;
			p_counter = COUNTER_ABABOL_PARP;
			p_ax = PLAYER_AX;
			p_rx = PLAYER_RX;
			p_salto = PLAYER_VY_INICIAL_SALTO;
			p_incr_salto = PLAYER_INCR_SALTO;	
		}	
	} else {
		// SWORD	
		if (cpc_TestKey (KEY_FIR)) {
			if (!s_on) {
				if (cpc_TestKey (KEY_ARR) && ! (cpc_TestKey (KEY_IZQ) || cpc_TestKey (KEY_DER))) {
					s_type = SWORD_TYPE_UP;
					s_next_frame = spr_21;
				} else {
					if (p_facing) {
						s_type = SWORD_TYPE_LEFT;
						s_next_frame = spr_20;
					} else {
						s_type = SWORD_TYPE_RIGHT;
						s_next_frame = spr_19;
					}
				}
				cpc_WyzStartEffect (WYZ_CANAL_EFECTOS, 4);
				s_on = 1;
				s_frame = 0;	
				p_swording = 1;
			}
		}	
				
		if (!possee || p_swording) {
			p_next_frame = player_frames [8 + p_facing];
		} else {
			it = p_facing << 2;
			if (p_vx == 0) {
				p_next_frame = player_frames [it + 1];
			} else {
				p_subframe ++;
				if (p_subframe == 4) {
					p_subframe = 0;
					p_frame = (p_frame + 1) & 3;
				}
				p_next_frame = player_frames [it + p_frame];
			}
		}
	}
}

signed int __FASTCALL__ abs (signed int n) {
	return n < 0 ? -n : n;
}

signed int __FASTCALL__ adjust_max (signed int i, signed int max) {
	return i < -max ? -max : i > max ? max : i;
}

void __FASTCALL__ move_enems (void) {
	for (eni = 0; eni < 3; eni ++) {
		enoffsmasi = enoffs + eni;
		if (malotes [enoffsmasi].t != 0) {
			switch (malotes [enoffsmasi].t) {
				case 6:
					// Boss
					if (en_an_state [eni] == EST_NORMAL) {
						if (p_x < en_an_x [eni]) en_an_vx [eni] -= 8;
						if (p_x > en_an_x [eni]) en_an_vx [eni] += 8;
						if (p_y < en_an_y [eni]) en_an_vy [eni] -= 8;
						if (p_y > en_an_y [eni]) en_an_vy [eni] += 8;
						
						en_an_vx [eni] = adjust_max (en_an_vx [eni], 256);
						en_an_vy [eni] = adjust_max (en_an_vy [eni], 256);
						
						en_an_x [eni] += en_an_vx [eni];
						en_an_y [eni] += en_an_vy [eni];
						
						if (en_an_x [eni] < 0) en_an_x [eni] = 0;
						if (en_an_x [eni] > 14336) en_an_x [eni] = 14336;
						if (en_an_y [eni] < 0) en_an_y [eni] = 0;
						if (en_an_y [eni] > 9216) en_an_y [eni] = 9216;
					}	
							
					ccx = en_an_x [eni] >> 6;
					ccy = en_an_y [eni] >> 6;
					
					en_an_count [eni] ++; 
					break;
				default:
					// Normal
					if (en_an_state [eni] != EST_PARALYZED) {
						malotes [enoffsmasi].x += malotes [enoffsmasi].mx;
						malotes [enoffsmasi].y += malotes [enoffsmasi].my;
						en_an_count [eni] ++; 
					} 
											
					ccx = malotes [enoffsmasi].x;
					ccy = malotes [enoffsmasi].y;
					
					if (ccx == malotes [enoffsmasi].x1 || ccx == malotes [enoffsmasi].x2)
						malotes [enoffsmasi].mx = -malotes [enoffsmasi].mx;
					if (ccy == malotes [enoffsmasi].y1 || ccy == malotes [enoffsmasi].y2)
						malotes [enoffsmasi].my = -malotes [enoffsmasi].my;
			}
				
			if (en_an_state [eni] != EST_NORMAL) {
				en_an_st_counter [eni] --;
				if (en_an_st_counter [eni] == 0) {
					if (en_an_state [eni] == EST_MUR) {
						malotes [enoffsmasi].t = 0;
						en_an_base_frame [eni] = spr_18;
						en_an_count [eni] = 0;
						en_an_frame [eni] = 0;
						p_objs ++;
						p_state = EST_NORMAL;
					} else {
						en_an_state [eni] = EST_NORMAL;
					}
				}
			}
				
			if (s_on) {
				if (s_frame > 2 && s_frame < 7) {
					if (s_x + 7 >= ccx && s_x <= ccx + 15 &&
						s_y + 7 >= ccy && s_y <= ccy + 15
					) {
						if (malotes [enoffsmasi].t < 6) {
							if (malotes [enoffsmasi].mx != 0)
								malotes [enoffsmasi].mx = sgni (ccx - p_px);
							if (malotes [enoffsmasi].my != 0)
								malotes [enoffsmasi].my = sgni (ccy - p_py);
							en_an_state [eni] = EST_PARALYZED;
							en_an_st_counter [eni] = COUNTER_MALO;
							cpc_WyzStartEffect (WYZ_CANAL_EFECTOS, 6);
						} else {
							if (en_an_state [eni] == EST_NORMAL) {
								men [enoffsmasi] --;
								if (men [enoffsmasi] == 0) {
									p_state = EST_STOP;
									en_an_state [eni] = EST_MUR;
									en_an_st_counter [eni] = COUNTER_MALO;
									en_an_base_frame [eni] = spr_17;
								} else {
									en_an_state [eni] = EST_PARP;
									en_an_st_counter [eni] = COUNTER_MALO;
								}
							}
							cpc_WyzStartEffect (WYZ_CANAL_EFECTOS, 6);
						}
					}
				}	
			}
			
			if (p_px + 12 >= ccx && p_px <= ccx + 12 &&
				p_py + 12 >= ccy && p_py <= ccy + 12 && 
				p_state == EST_NORMAL && en_an_state [eni] != EST_PARP
			) {
				cpc_WyzStartEffect (WYZ_CANAL_EFECTOS, 7);
				decrease_life ();
				p_state = EST_ICE;
				p_counter = COUNTER_FRIGOABABOL;
				p_ax = p_rx = 4;
				p_salto = 32;
				p_incr_salto = 0;
			}
					
			if (en_an_count [eni] == 4) {
				en_an_count [eni] = 0;
				en_an_frame [eni] = !en_an_frame [eni];
			}
				
			if (en_an_state [eni] == EST_PARP && half_life) {
				en_an_next_frame [eni] = spr_18;
			} else {
				it = en_an_frame [eni];
				en_an_next_frame [eni] = en_an_base_frame [eni] + ((it << 6) + (it << 1));
			}
		}
	}
}

unsigned char res = 0;
unsigned char game () {
	res = 0;
	y_pant = 19;

	load_level ();	
	init_game ();
	init_sprites ();
	
	o_life = 99;
	o_score = 99;
	o_keys = 99;

	render_screen ();
	
	do_process_breakable = 0;

	// Poner msica ingame
	if (level) play_song (song_2); else play_song (song_1);
		p_vx=0;
	while (1) {
		if (o_keys != p_keys) {
			print_number2 (19, 22, p_keys);
			o_keys = p_keys;
		}
		if (o_score != p_score) {
			print_number3 (13, 22, p_score);
			o_score = p_score;
		}
		if (o_life != p_life) {
			print_number2 (8, 22, p_life);
			o_life = p_life;
		}
		
		half_life = 1 - half_life;
		
		if (p_state != EST_STOP) move ();
		move_enems ();
		
		if (do_process_breakable) process_breakable ();
		
		// Cambio de pantalla
		if (p_y == 0) {
			if (p_state != EST_ICE && p_vy < 0 && y_pant > 0) {
				y_pant --;
				render_screen ();
				p_y = 9216;	
			}
		}
		if (p_y == 9216) {
			if (p_vy > 0 && y_pant < 19) {				// 9216 = 144 * 64
				y_pant ++;
				render_screen ();
				p_y = 0;
				if (p_vy > 256) p_vy = 256;
			}
		}
		
		// Render
		if (s_on) render_sword ();
		
		if (p_state == EST_PARP) {
			if (half_life)
				p_next_frame = spr_18;
			p_counter --;
			if (p_counter == 0)
				p_state = EST_NORMAL;	
		}
		
		sp_player.sp0 = (int) (p_next_frame);
		sp_player.cx = p_x >> 8;
		sp_player.cy = p_y >> 6;
		
		if (malotes [enoffs].t < 6) {
			px = malotes [enoffs].x >> 2;
			py = malotes [enoffs].y;
		} else {
			px = en_an_x [0] >> 8;	
			py = en_an_y [0] >> 6;
		}
		sp_enem1.sp0 = (int) (en_an_next_frame [0]);
		sp_enem1.cx = px;
		sp_enem1.cy = py;	
	
		if (malotes [enoffs + 1].t < 6) {
			px = malotes [enoffs + 1].x >> 2;
			py = malotes [enoffs + 1].y;
		} else {
			px = en_an_x [1] >> 8;	
			py = en_an_y [1] >> 6;
		}
		sp_enem2.sp0 = (int) (en_an_next_frame [1]);
		sp_enem2.cx = px;
		sp_enem2.cy = py;
		
		if (malotes [enoffs + 2].t < 6) {
			px = malotes [enoffs + 2].x >> 2;
			py = malotes [enoffs + 2].y;
		} else {
			px = en_an_x [2] >> 8;	
			py = en_an_y [2] >> 6;
		}
		sp_enem3.sp0 = (int) (en_an_next_frame [2]);
		sp_enem3.cx = px;
		sp_enem3.cy = py;	
		
		if (s_on) {
			sp_sword.sp0 = (int) (s_next_frame);
		} else {
			sp_sword.sp0 = (int) (spr_18);
		}
		sp_sword.cx = s_x >> 2;
		sp_sword.cy = s_y;

		cpc_PutSpTileMap ((int) (sp_sword));
		cpc_PutSpTileMap ((int) (sp_player));
		cpc_PutSpTileMap ((int) (sp_enem1));
		cpc_PutSpTileMap ((int) (sp_enem2));
		cpc_PutSpTileMap ((int) (sp_enem3));
		
		cpc_UpdScr ();
		
		// Mode 0
		cpc_PutTrSp8x16TileMap2b ((int) (sp_sword));
		cpc_PutTrSp8x16TileMap2b ((int) (sp_player));
		cpc_PutTrSp8x16TileMap2b ((int) (sp_enem1));
		cpc_PutTrSp8x16TileMap2b ((int) (sp_enem2));
		cpc_PutTrSp8x16TileMap2b ((int) (sp_enem3));
		
		// Update		
		cpc_ShowTouchedTiles ();
		cpc_ResetTouchedTiles ();
	
		// Exit?
		if (p_life == 0) break;
		if (p_objs == 1) { res = 1; break; }
		
		// Exit
		if (cpc_TestKey (KEY_ESC)) break; 
	}

	// Parar msica ingame
	cpc_WyzSetPlayerOff ();
	
	return res;
}

