// levelmanager.h
// This module unpacks levels from RAMx, sets them up, and renders screens.

#define BUFFER 10000
#define ATTR(x,y)	map_attr[x+(y<<4)-y]
#define TILEAT(x,y) map_buff[x+(y<<4)-y]

// Compressed levels
// Levels are map + enems, compressed.
extern unsigned char clevel4 [0];
extern unsigned char clevel5 [0];
#asm
	._clevel4
		BINARY "level4c.exo"
	._clevel5
		BINARY "level5c.exo"
#endasm
unsigned char *clevels [] = {
	clevel4, clevel5
};

unsigned char *mapa = (unsigned char *) (BUFFER);
MALOTE *malotes = (MALOTE *) (BUFFER + 1500);
unsigned char map_buff [150];
unsigned char map_attr [150];
unsigned char y_pant;

// Add these:
// 0 = nothing
// 1 = kills
// 2 = kills (area)
// 4 = platform
// 8 = obstacle
// 16 = breakable

unsigned char tile_behaviour [32] = {
	0, 8, 8, 4, 0, 8, 8, 8, 8, 4, 0,24, 1, 8, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

unsigned char music_track = 0;

void load_level () {
	// Unpack level
	cpc_UnExo ((unsigned int *) clevels [level], (unsigned int *) (BUFFER));
}

void srand (void);
void render_screen () {
	print_number2 (24, 22, y_pant);
	seed1 [0] = y_pant; seed2 [0] = level;
	srand ();

	map_pointer = (unsigned char *) (BUFFER + (y_pant * 75));
	gpx = gpy = 0;
	for (it = 0; it < 150; it ++) {
		if (0 == (it & 1)) {
			gpc = *map_pointer ++;
			gpt = gpc >> 4;
		} else {
			gpt = gpc & 15;
		}
		map_buff [it] = gpt;
		map_attr [it] = tile_behaviour [gpt];
		
		draw_map_tile (gpx, gpy, gpt);
		
		gpx ++;
		if (gpx == 15) {
			gpy ++;
			gpx = 0;
		}
	}
	
	cpc_ResetTouchedTiles ();	
	cpc_ShowTileMap (0);
	
	// Create enemies:
	enoffs = y_pant * 3;
	
	for (it = 0; it < 3; it ++) {
		en_an_frame [it] = 0;
		en_an_count [it] = 0;
		
		// If defined, frame = sprite_9_a + (2 frames) * (type - 1).
		switch (malotes [enoffs + it].t) {
			case 0:
				en_an_base_frame [it] = spr_18;
				break;
			case 1:	
			case 2:
			case 3:
			case 4:
				en_an_base_frame [it] = spr_8 + 132 * (malotes [enoffs + it].t - 1);
				break;
			case 6:
				en_an_base_frame [it] = spr_14;
				break;
		}
		en_an_next_frame [it] = en_an_base_frame [it];
		
		en_an_state [it] = EST_NORMAL;
		en_an_x [it] = malotes [enoffs + it].x << 6;
		en_an_y [it] = malotes [enoffs + it].y << 6;
		en_an_vx [it] = en_an_vy [it] = 0;
	}
}

void set_map_tile (unsigned char x, unsigned char y, unsigned char tile) {
	// Map is packed, so we firstly calculate an unpacked
	// address, then divide by two, then mask out/replace the
	// correct nibble, &1 = 0 MSN, &1 = 1 LSN.

	tindex = 150 * y_pant + 15 * y + x;	
	map_pointer = (unsigned char *) (BUFFER + (tindex >> 1));
	if ((tindex & 1) == 0) {
		*map_pointer = (*map_pointer & 0x0f) | (tile << 4);
	} else {
		*map_pointer = (*map_pointer & 0xf0) | (tile & 15);
	}
}
